--------------------------------------------------------------------------------
local model, config, pages, rows, columns, texts, formID, loaded, saved
local windows = 2
local text = ""
local extension = ".txt"
local version = "1.0"

--------------------------------------------------------------------------------
local function showPage(window)
	local r,g,b   = lcd.getBgColor()
	local font    = FONT_MINI
	local startX  = 0
	local startY  = 1
	local offset  = 0
	local border  = 2
	local height  = lcd.getTextHeight(font, "|") + border*2
	local rows    = rows[window]
	local columns = columns[window]
	local texts   = texts[window]
	
	if (r+g+b)/3 > 128 then
	    r,g,b = 0,0,0
	else
	    r,g,b = 255,255,255
	end
	
	lcd.setColor(r,g,b)
	
	for j=1, columns do
		local width = 0
		
		for i=1, rows do
			local currentWidth = lcd.getTextWidth(font, texts[i][j]) + border*2
			if (width < currentWidth) then
				width = currentWidth
			end
		end 
		
		if (j > 1) then
			local x = startX+offset
			lcd.drawLine(x, startY, x, startY+height*rows)
		end		
		
		for i=1, rows do
			lcd.drawText(startX+offset+border, startY+height*(i-1)+border, texts[i][j], font)
		end
		
		offset = offset + width
	end
	
	for i=1, rows do
		if (i > 1) then
			local y = startY+height*(i-1)
			lcd.drawLine(startX, y, startX+offset, y)
		end
	end
	
	lcd.drawRectangle(startX, startY, offset+1, height*rows+1)
end

---------------------------------------------------------------------------------
local function showPage1()
	return showPage(1)
end

---------------------------------------------------------------------------------
local function showPage2()
	return showPage(2)
end

---------------------------------------------------------------------------------
local function setupForm1()
	for w=1, windows do		
		form.addRow(1)
		form.addLabel({label = "Fenster "..w, font=FONT_BOLD})
		
		form.addRow(2)
		form.addLabel({label = "Zeilen", width=200})
		form.addIntbox(rows[w], 1, 10, 2, 0, 1, function(value)
			if (rows[w] < value) then
				for i=rows[w]+1, value do
					texts[w][i] = {}
					for j=1, columns[w] do
						texts[w][i][j] = text
						system.pSave("text."..w.."."..i.."."..j, text)
					end
				end
			else
				for i=value+1, rows[w] do
					texts[w][i] = nil
					for j=1, columns[w] do
						system.pSave("text."..w.."."..i.."."..j, nil)
					end
				end
			end
		
			rows[w] = value
			system.pSave("row."..w, value)
		end)
		
		form.addRow(2)
		form.addLabel({label = "Spalten", width=200})
		form.addIntbox(columns[w], 1, 10, 2, 0, 1, function(value)
			if (columns[w] < value) then
				for i=1, rows[w] do
					for j=columns[w]+1, value do
						texts[w][i][j] = text
						system.pSave("text."..w.."."..i.."."..j, text)
					end
				end
			else
				for i=1, rows[w] do
					for j=value+1, columns[w] do
						texts[w][i][j] = nil
						system.pSave("text."..w.."."..i.."."..j, nil)
					end
				end
			end

			columns[w] = value
			system.pSave("column."..w, value)
		end)
		
		form.addSpacer(1, 7)
	end
	
	form.addRow(2)
  form.addLabel({label = "Name", width=200})
	form.addTextbox(config, 63, function(value)
		config = value
		system.pSave("config", value)
		form.setButton(4, "S", config:len() > 0 and ENABLED or DISABLED)
		form.setButton(5, "L", config:len() > 0 and ENABLED or DISABLED)
	end)
	
	form.addRow(1)
	form.addLabel({label="Powered by Thorn - v."..version, font=FONT_MINI, alignRight=true})
end

---------------------------------------------------------------------------------
local function setupFormTable(window)
	local rows = rows[window]
	local columns = columns[window]
	local texts = texts[window]

	for i=1, rows do
		if (i > 1) then
			form.addSpacer(1, 7)
		end
		
		form.addRow(1)
		form.addLabel({label = "Zeile "..i, font=FONT_BOLD})
		local row = texts[i]
	
		for j=1, columns do
			form.addRow(2)
			form.addLabel({label = "Spalte "..j, width=200})
			form.addTextbox(row[j], 63, function(value)				
				row[j] = value
				system.pSave("text."..window.."."..i.."."..j, value)
			end)
		end
	end
end

---------------------------------------------------------------------------------
local function setupForm2()
	setupFormTable(1)
end

---------------------------------------------------------------------------------
local function setupForm3()
	setupFormTable(2)
end

---------------------------------------------------------------------------------
local function setupForm(id)
	formID = id
	
	if (formID == 1) then
		setupForm1()
	elseif (formID == 2) then
		setupForm2()
	elseif (formID == 3) then
		setupForm3()
	end
	
	form.setButton(1, "O", formID == 1 and HIGHLIGHTED or ENABLED)
	form.setButton(2, "1", formID == 2 and HIGHLIGHTED or ENABLED)
	form.setButton(3, "2", formID == 3 and HIGHLIGHTED or ENABLED)
	form.setButton(4, "S", formID == 1 and saved  and HIGHLIGHTED or formID == 1 and config:len() > 0 and ENABLED or DISABLED)
	form.setButton(5, "L", formID == 1 and loaded and HIGHLIGHTED or formID == 1 and config:len() > 0 and ENABLED or DISABLED)
end

---------------------------------------------------------------------------------
local function saveConfig()
	if (config:len() > 0) then
		local file = io.open(config..extension, "w+")
		if (file) then
			local row = ""
			local column = ""
			local text = ""
			
			for w=1, windows do
				if (w > 1) then
					row    = row.." "
					column = column.." "
					text   = text.." "
				end
			
				row    = row..rows[w]
				column = column..columns[w]
				
				for i=1, rows[w] do
					if (i > 1) then
						text = text.." "
					end
					for j=1, columns[w] do
						if (j > 1) then
							text = text.." "
						end
						text = text.."\""..texts[w][i][j].."\""
					end
				end
			end	
					
			io.write(file, row.."\n")
			io.write(file, column.."\n")
			io.write(file, text.."\n")
			io.close(file)
			
			saved = system.getTimeCounter()
		end
	end
end

---------------------------------------------------------------------------------
local function loadConfig()
	if (config:len() > 0) then		
		local file = io.open(config..extension, "r")
		if (file) then
			local row = {}
			local column = {}
			local text = {}
			local count = 0
			local line, index
			
			repeat
				line = io.readline(file, true)
				if (not line) then
					io.close(file)
					return
				end
				
				count = count + 1
				index = 0
				
				if (count == 1) then
					for value in line:gmatch("%S+") do
						index = index + 1
						row[index] = tonumber(value)
					end
				elseif (count == 2) then
					for value in line:gmatch("%S+") do
						index = index + 1
						column[index] = tonumber(value)
					end
				elseif (count == 3) then
					local w,i,j = 1,1,1
					for value in line:gmatch("\"([^\"]*)\"") do
						if (not text[w]) then
							text[w] = {}
						end
						
						if (not text[w][i]) then
							text[w][i] = {}
						end
						
						text[w][i][j] = value
						
						j = j + 1
						if (j > column[w]) then
							j = 1
							i = i + 1
							
							if (i > row[w]) then
								i = 1
								w = w + 1
								if (w > windows) then
									break
								end
							end
						end
					end
				end
			until (count >= 3)
			
			for w=1, windows do
				system.pSave("row."..w, row[w])
				system.pSave("column."..w, column[w])
				
				for i=1, row[w] do
					for j=1, column[w] do
						system.pSave("text."..w.."."..i.."."..j, text[w][i][j])
					end
				end
			end

			rows = row
			columns = column
			texts = text
		
			io.close(file)
			loaded = system.getTimeCounter()
		end
	end
end

---------------------------------------------------------------------------------
local function keyForm(key)
	if (key == KEY_1 and formID ~= 1) then
		form.reinit(1)
	elseif (key == KEY_2 and formID ~= 2) then
		form.reinit(2)
	elseif (key == KEY_3 and formID ~= 3) then
		form.reinit(3)
	elseif (key == KEY_4 and formID == 1) then
		saveConfig()
		form.reinit(1)
	elseif (key == KEY_5 and formID == 1) then
		form.preventDefault()
		loadConfig()
		form.reinit(1)
	end
end

---------------------------------------------------------------------------------
local function loop()
	if (saved or loaded) then
		local time  = system.getTimeCounter()
		local limit = 1000
		if (saved and time - saved > limit) then
			saved = nil
			form.setButton(4, "S", config:len() > 0 and ENABLED or DISABLED)
		end
		
		if (loaded and time - loaded > limit) then
			loaded = nil
			form.setButton(5, "L", config:len() > 0 and ENABLED or DISABLED)
		end
	end
end

---------------------------------------------------------------------------------
local function init()
	pages = {showPage1, showPage2}
	model = system.getProperty("Model") or ""	
	config = system.pLoad("config", "")	
	rows = {}
	columns = {}
	texts = {}
	
	for w=1, windows do	
		local r = system.pLoad("row."..w, 2)
		local c = system.pLoad("column."..w, 2)
		
		local win = {}
		for i=1, r do
			local row = {}
			for j=1, c do
					row[j] = system.pLoad("text."..w.."."..i.."."..j, text)
			end
			win[i] = row
		end
		texts[w] = win
		rows[w] = r
		columns[w] = c
	end
	
	system.registerForm(1, MENU_APPS, "Schalterbelegung", setupForm, keyForm)
	for w=1, windows do
		system.registerTelemetry(w, "Schalterbelegung "..w.." - "..model, 4, pages[w])   -- full size Window
	end
end
--------------------------------------------------------------------------------

return {init=init, loop=loop, author="Thorn", version=version, name="Schalter"}